package pipe.gui.widgets.cpn;

import dk.aau.cs.debug.Logger;
import dk.aau.cs.gui.Context;
import dk.aau.cs.model.tapn.Colored.*;
import dk.aau.cs.model.tapn.Colored.Expressions.*;
import pipe.gui.Grid;
import pipe.gui.graphicElements.PetriNetObject;
import pipe.gui.graphicElements.Place;
import pipe.gui.graphicElements.cpn.ColoredInputArcComponent;
import pipe.gui.graphicElements.cpn.ColoredOutputArcComponent;
import pipe.gui.graphicElements.cpn.ColoredTransportArcComponent;
import pipe.gui.widgets.GuardDialogue;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

public class ColoredArcDialogPanel extends GuardDialogue {

    private ColorIntervalAndInvariantEditPanel colorIntervalPanel;
    JTabbedPane exprTabbedPane;
    private JSpinner colorExpressionWeightSpinner;
    private JLabel weightLabel;
    private JPanel transportWeightPanel;

    private int transportWeight;
    private ColorExpression transportInputExpr;
    private ColorExpression transportOutputExpr;

    private ColoredInputArcComponent objectToBeEditedInput;
    private ColoredOutputArcComponent objectToBeEditedOutput;
    private ColoredTransportArcComponent objectToBeEditedTransport;
    private boolean input;
    private boolean transportArc;
    private Context context;
    private PetriNetObject petriNetObject;
    private PetriNetObject objectToBeEdited;


    public ColoredArcDialogPanel(JRootPane rootPane, PetriNetObject objectToBeEdited, Context context) {
        super(rootPane, objectToBeEdited);
        this.context = context;
        this.objectToBeEdited = objectToBeEdited;

        if (objectToBeEdited instanceof ColoredTransportArcComponent) {
            if (((ColoredTransportArcComponent) objectToBeEdited).getSource() instanceof Place) {
                this.objectToBeEditedTransport = (ColoredTransportArcComponent)objectToBeEdited;
                input = true;
                transportArc = true;
            }
            else {
                this.objectToBeEditedTransport = (ColoredTransportArcComponent)objectToBeEdited;
                input = false;
                transportArc = true;
            }
        }
        else if (objectToBeEdited instanceof ColoredInputArcComponent) {
            this.objectToBeEditedInput = (ColoredInputArcComponent) objectToBeEdited;
            input = true;
            transportArc = false;
        } else if (objectToBeEdited instanceof ColoredOutputArcComponent) {
            this.objectToBeEditedOutput = (ColoredOutputArcComponent) objectToBeEdited;
            transportArc = false;
            input = false;
        }
        petriNetObject = objectToBeEdited;
        initComponents();
    }

    private void initComponents() {
        if (transportArc)
            setTransportExpression();
        initPanels();
        initonOK();
    }

    private void initonOK() {
        okButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent actionEvent) {
                onOkColored();
            }
        });
    }

    private void setTransportExpression() {
        ArcExpression arcExprInput = ((ColoredTransportArc)objectToBeEditedTransport.underlyingTransportArc()).getInputExpression();
        ArcExpression arcExprOutput = ((ColoredTransportArc)objectToBeEditedTransport.underlyingTransportArc()).getOutputExpression();

        if (arcExprInput instanceof NumberOfExpression) {
            transportWeight = arcExprInput.weight();
            Vector<ColorExpression> vecColorExpr = ((NumberOfExpression) arcExprInput).getColor();
            transportInputExpr = new TupleExpression(vecColorExpr);
        }

        if (arcExprOutput instanceof  NumberOfExpression) {
            Vector<ColorExpression> vecColorExpr = ((NumberOfExpression) arcExprOutput).getColor();
            transportOutputExpr = new TupleExpression(vecColorExpr);
        }
    }

    private ArcExpression getTransportExpression(ColorExpression colorExpr, int weight) {
        ArcExpression expr;
        Vector<ColorExpression> vecColorExpr = new Vector<ColorExpression>();
        if (colorExpr instanceof TupleExpression) { // we have to use TupleExpression if we want the colorExpressionPanel inside arcPanel when it is transport. IF the tuple only have one element we extract it to remove an unnecessary expression and parentheses
            if (((TupleExpression) colorExpr).getColors().size() == 1) {
                colorExpr = ((TupleExpression) colorExpr).getColors().firstElement();
            }
        }
        vecColorExpr.add(colorExpr);
        expr = new NumberOfExpression(weight, vecColorExpr);
        return expr;
    }

    private void onOkColored() {
        if (input) {
            if (transportArc) {
                int weight =  Integer.parseInt(colorExpressionWeightSpinner.getValue().toString());
                ColoredTransportArc transportArc = (ColoredTransportArc)objectToBeEditedTransport.underlyingTransportArc();
                ArcExpression inputExpression = getTransportExpression(((ColorExpressionDialogPanel)exprTabbedPane.getComponentAt(0)).getColorExpression(), weight);
                ArcExpression outputExpression = getTransportExpression(((ColorExpressionDialogPanel)exprTabbedPane.getComponentAt(1)).getColorExpression(), weight);
                transportArc.setInputExpression(inputExpression);
                transportArc.setOutputExpression(outputExpression);
                objectToBeEditedTransport.setUnderlyingArc(transportArc);
                ((ColoredTransportArc)((ColoredTransportArcComponent)petriNetObject).underlyingTransportArc()).setInputExpression(inputExpression);
                ((ColoredTransportArc)((ColoredTransportArcComponent)petriNetObject).underlyingTransportArc()).setOutputExpression(outputExpression);
                ((ColoredTransportArc)((ColoredTransportArcComponent)petriNetObject).underlyingTransportArc()).setCtiList(getctiList());
                objectToBeEditedTransport.updateLabel(false);
            } else {
                ColoredInputArc inputArc = (ColoredInputArc)objectToBeEditedInput.underlyingTimedInputArc();
                ArcExpression arcExpression = ((ArcExpressionPanel)exprTabbedPane.getComponentAt(0)).getExpression();
                inputArc.setExpression(arcExpression);
                objectToBeEditedInput.setUnderlyingArc(inputArc);
                ((ColoredInputArc)((ColoredInputArcComponent)petriNetObject).underlyingTimedInputArc()).setExpression(arcExpression);
                ((ColoredInputArc)((ColoredInputArcComponent)petriNetObject).underlyingTimedInputArc()).setColorTimeIntervals(getctiList());
                objectToBeEditedInput.updateLabel(false);
            }
        } else {
            ArcExpression arcExpression = ((ArcExpressionPanel)exprTabbedPane.getComponentAt(0)).getExpression();
            ColoredOutputArc outputArc = (ColoredOutputArc)objectToBeEditedOutput.underlyingArc();
            objectToBeEditedOutput.setUnderlyingArc(outputArc);
            ((ColoredOutputArc)((ColoredOutputArcComponent)petriNetObject).underlyingArc()).setExpression(arcExpression);
        }

    }

    private List<ColoredTimeInterval> getctiList() {
        List<ColoredTimeInterval> ctiList = new ArrayList<ColoredTimeInterval>();
        for (int i = 0; i < colorIntervalPanel.getTimeConstraintModel().size(); i++) {
            ctiList.add((ColoredTimeInterval) colorIntervalPanel.getTimeConstraintModel().get(i));
        }
        return ctiList;
    }

    private ColorExpression getColorExpression(ArcExpression arcExpr) {
        ColorExpression colorExpression = null;
        if (arcExpr instanceof NumberOfExpression) {
            colorExpression = new TupleExpression(((NumberOfExpression) arcExpr).getColor());
        }
        return colorExpression;
    }

    private void initPanels() {
        exprTabbedPane = new JTabbedPane();
        if(input) {
            if (transportArc) {
                transportWeightPanel = new JPanel(new GridBagLayout());
                int current = transportWeight;
                int min = 1;
                int max = 9999;
                int step = 1;
                SpinnerNumberModel numberModel = new SpinnerNumberModel(current, min, max, step);
                colorExpressionWeightSpinner = new JSpinner(numberModel);
                weightLabel = new JLabel("Weight:");
                ColorExpressionDialogPanel inputPanel = new ColorExpressionDialogPanel(getRootPane(), context, transportInputExpr, true);
                exprTabbedPane.add(inputPanel, "input");
            } else {
                ArcExpressionPanel inputPanel = new ArcExpressionPanel(objectToBeEdited, context, true);
                exprTabbedPane.add(inputPanel, "input");
            }
        }else {
            ArcExpressionPanel inputPanel = new ArcExpressionPanel(objectToBeEdited, context, true);
            exprTabbedPane.add(inputPanel, "output");
        }
        if (transportArc) {
            ColorExpressionDialogPanel outputPanel = new ColorExpressionDialogPanel(getRootPane(), context, transportOutputExpr, true);
            exprTabbedPane.add(outputPanel, "output");
        }
        if (input)
            initColoredTimedGuard();

        if (transportArc) {
            GridBagConstraints gbc = new GridBagConstraints();
            gbc.gridx = 0;
            gbc.gridy = 0;
            gbc.insets = new Insets(5, 10 ,5, 10);
            gbc.anchor = GridBagConstraints.WEST;
            transportWeightPanel.add(weightLabel, gbc);

            gbc.gridx = 1;
            transportWeightPanel.add(colorExpressionWeightSpinner, gbc);

            gbc.gridx = 0;
            gbc.gridy = 2;
            gbc.anchor = GridBagConstraints.WEST;
            add(transportWeightPanel, gbc);
        }

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;

        if (!transportArc)
            gbc.gridy = 2;
        else
            gbc.gridy = 3;

        gbc.insets = new Insets(5, 10, 5, 10);
        gbc.fill = GridBagConstraints.BOTH;
        gbc.anchor = GridBagConstraints.WEST;
        add(exprTabbedPane, gbc);
    }

    private void initColoredTimedGuard() {
        ColorType ct;
        if (!transportArc) {
            if(objectToBeEditedInput.underlyingTimedInputArc().source() instanceof SharedColoredPlace) {
                ct = ((SharedColoredPlace)objectToBeEditedInput.underlyingTimedInputArc().source()).getColorType();
                colorIntervalPanel = new ColorIntervalAndInvariantEditPanel(context, ct, true, objectToBeEditedInput);
            }else {
                ct = ((LocalColoredPlace) objectToBeEditedInput.underlyingTimedInputArc().source()).getColorType();
                colorIntervalPanel = new ColorIntervalAndInvariantEditPanel(context, ct, true, objectToBeEditedInput);
            }
        }
        else {
            if(objectToBeEditedTransport.underlyingTransportArc().source() instanceof  SharedColoredPlace) {
                ct = (((SharedColoredPlace) objectToBeEditedTransport.underlyingTransportArc().source()).getColorType());
                colorIntervalPanel = new ColorIntervalAndInvariantEditPanel(context, ct, true, objectToBeEditedInput);
            }else {
                ct = ((LocalColoredPlace) objectToBeEditedTransport.underlyingTransportArc().source()).getColorType();
                colorIntervalPanel = new ColorIntervalAndInvariantEditPanel(context, ct, true, objectToBeEditedTransport);
            }
        }

        GridBagConstraints gbc = new GridBagConstraints();
        gbc.gridx = 0;
        gbc.gridy = 0;
        gbc.insets = new Insets(5, 10, 5, 10);
        gbc.fill = GridBagConstraints.BOTH;
        gbc.anchor = GridBagConstraints.WEST;
        add(colorIntervalPanel, gbc);
    }
}
